use crate::filter_expr::opcode::OpCode;

/// Convert:
///     LOAD reg var
///     MOV target reg
/// To:
///     LOAD target var
pub fn remove_redundant_load_and_mov(mut opcodes: Vec<OpCode>) -> (Vec<OpCode>, bool) {
    let old_total_opcodes = opcodes.len();
    let new_opcodes = opcodes.into_iter().fold(Vec::new(), |mut acc, current| {
        if let OpCode::MOV(target, from) = &current {
            if let Some(last) = acc.last_mut() {
                match last {
                    OpCode::LOAD(reg, _) => {
                        if reg == from {
                            *reg = *target;
                            return acc;
                        }
                    }
                    _ => {}
                }
            }
        }
        acc.push(current);
        acc
    });
    let new_total_opcodes = new_opcodes.len();

    (new_opcodes, old_total_opcodes != new_total_opcodes)
}
