use super::{expr::Variable, Error, Filter, Value};
use crate::filter_expr::value::ValueOrRef;
use std::collections::HashMap;

fn external_variables<K: Into<Variable>, V: Into<Value>>(
    external_variables: Vec<(K, V)>,
) -> HashMap<Variable, ValueOrRef<'static>> {
    external_variables
        .into_iter()
        .map(|(k, v)| (k.into(), ValueOrRef::Value(v.into())))
        .collect()
}

fn testsuite<K: Into<Variable>, V: Into<Value>, R: Into<Value>>(
    code: &str,
    variables: Vec<(K, V)>,
    ret: Result<R, Error>,
) -> Filter {
    let filter = Filter::new(code).expect("valid filter");
    let variables = external_variables(variables);
    println!("{}\n\n{}", filter.debug(), filter.dump());
    assert_eq!(filter.execute(&variables), ret.map(|x| x.into()));
    filter
}

#[test]
fn test_1() {
    testsuite(
        include_str!("program1.expr"),
        vec![("external", 5)],
        Ok(true),
    );
    testsuite(
        include_str!("program1.expr"),
        vec![("external", 4)],
        Ok(false),
    );
}

#[test]
fn nested() {
    let vm = testsuite(
        include_str!("nested.expr"),
        vec![("foo", 0), ("bar", 0), ("five", 5)],
        Ok(false),
    );

    assert_eq!(
        vm.execute(&external_variables(vec![
            ("foo", 1192844),
            ("bar", 1),
            ("five", 4),
        ])),
        Ok(false.into())
    );

    assert_eq!(
        vm.execute(&external_variables(vec![
            ("foo", 1192844),
            ("bar", 1),
            ("five", 5),
        ])),
        Ok(true.into())
    );
}

#[test]
fn missing_external_variable() {
    testsuite::<_, _, bool>(
        include_str!("nested.expr"),
        vec![("foo", 1192844), ("bar", 1)],
        Err(Error::VariableNotFound("five".to_owned())),
    );
}

#[test]
fn rearrange_expr_from_less_expensive() {
    testsuite::<&str, i128, _>(include_str!("always_true.expr"), vec![], Ok(true));
}
